import factory

from .models import EncryptedTextFieldModel


class EncryptedTextFieldModelFactory(factory.DjangoModelFactory):
    """Factory to generate `encrypted_value`."""
    class Meta:
        model = EncryptedTextFieldModel

    encrypted_value = factory.Sequence('Text {}'.format)



from django.test import TestCase
from django.test.utils import override_settings

from pgcrypto_fields import aggregates, fields
from .factories import EncryptedTextFieldModelFactory
from .models import EncryptedTextFieldModel


PUBLIC_PGP_KEY = 'my public key'


class TestEncryptedTextField(TestCase):
    """Test `EncryptedTextField` behave properly."""
    field = fields.EncryptedTextField

    def test_db_type(self):
        """Check db_type is `bytea`."""
        self.assertEqual(self.field().db_type(), 'bytea')

    @override_settings(PUBLIC_PGP_KEY=PUBLIC_PGP_KEY)
    def test_get_placeholder(self):
        """Check `get_placeholder` returns the right string function to encrypt data."""
        expected = "pgp_pub_encrypt(%s, dearmor('{}'))".format(PUBLIC_PGP_KEY)
        self.assertEqual(self.field().get_placeholder(), expected)

    def test_south_field_triple(self):
        """Check return a suitable description for south migration."""
        south_triple = fields.EncryptedTextField().south_field_triple()
        expected = ('pgcrypto_fields.fields.EncryptedTextField', [], {})
        self.assertEqual(south_triple, expected)


class TestEncryptedTextFieldModel(TestCase):
    """Test `EncryptedTextField` can be integrated in a `Django` model."""
    model = EncryptedTextFieldModel

    def test_fields(self):
        """Assert fields are representing our model."""
        fields = self.model._meta.get_all_field_names()
        expected = ('id', 'encrypted_value')
        self.assertCountEqual(fields, expected)

    def test_value_returned_is_bytea(self):
        """Assert value returned is a memoryview instance."""
        EncryptedTextFieldModelFactory.create()

        instance = EncryptedTextFieldModel.objects.get()
        self.assertIsInstance(instance.encrypted_value, memoryview)

    def test_value(self):
        """Assert we can get back the decrypted value."""
        expected = 'bonjour'
        EncryptedTextFieldModelFactory.create(encrypted_value=expected)

        queryset = EncryptedTextFieldModel.objects.annotate(
            aggregates.Decrypt('encrypted_value'),
        )
        instance = queryset.get()
        self.assertEqual(instance.encrypted_value__decrypt, expected)



from django.conf import settings
from django.db import models


class DecryptFunctionSQL(models.sql.aggregates.Aggregate):
    """Custom SQL aggregate to decrypt a field.

    `DecryptFunctionSQL` provides a SQL template using pgcrypto to decrypt
    data from a field in the database.

    `sql_function` defines `pgp_pub_decrypt` which is a pgcrypto SQL function.
    This function takes two arguments:
      - a encrypted message (bytea);
      - a key (bytea).

    `%(function)s` in `sql_template` is populated by `sql_function`.

    `%(field)s` is replaced with the field's name.

    `dearmor` is used to unwrap the key from the PGP key.
    """
    sql_function = 'pgp_pub_decrypt'
    sql_template = "%(function)s(%(field)s, dearmor('{}'))".format(
        settings.PRIVATE_PGP_KEY,
    )


class Decrypt(models.Aggregate):
    """`Decrypt` creates an alias for `DecryptFunctionSQL`.

    `alias` is `{self.lookup}__decrypt` where 'decrypt' is `self.name.lower()`.

    `self.lookup` is defined in `models.Aggregate.__init__`.
    """
    name = 'Decrypt'

    def add_to_query(self, query, alias, col, source, is_summary):
        """Add the aggregate to the query."""
        aggregate = DecryptFunctionSQL(
            col,
            source=source,
            is_summary=is_summary,
            **self.extra
        )
        query.aggregates[alias] = aggregate



from django.conf import settings
from django.db import models


class EncryptedTextField(models.TextField):
    """Encrypted TextField.

    `EncryptedTextField` deals with postgres and use pgcrypto to encode
    data to the database. Compatible with django 1.6.x for migration.
    """
    def db_type(self, connection=None):
        """Value stored in the database is hexadecimal."""
        return 'bytea'

    def get_placeholder(self, value=None, connection=None):
        """
        Tell postgres to encrypt this field with our public pgp key.

        `%s` is replaced with the field's value.

        `value` and `connection` are ignored here as we don't need other custom
        operator depending on the value.

        `pgp_pub_encrypt` and `dearmor` are `pgcrypto` functions which encrypt
        the field's value with the PGP key unwrapped by `dearmor`.
        """
        return "pgp_pub_encrypt(%s, dearmor('{}'))".format(settings.PUBLIC_PGP_KEY)

    def south_field_triple(self):
        """Return a suitable description of this field for South."""
        from south.modelsinspector import introspector
        field_class = '{}.{}'.format(self.__class__.__module__, self.__class__.__name__)
        args, kwargs = introspector(self)
        return (field_class, args, kwargs)



from setuptools import find_packages, setup


version = '0.1.0'


setup(
    name='django-pgcrypto-fields',
    packages=find_packages(),
    include_package_data=True,
    version=version,
    license='BSD',
    description='Encrypted fields dealing with pgcrypto postgres extension.',
    author='Incuna Ltd',
    author_email='admin@incuna.com',
    url='https://github.com/incuna/django-pgcrypto-fields',
)



from django.db import models

from pgcrypto_fields.fields import EncryptedTextField


class EncryptedTextFieldModel(models.Model):
    """Dummy model used for tests to check `EncryptedTextField`."""
    encrypted_value = EncryptedTextField()
